import { DominoesGame } from "./DominoesGame";
import { MatchSession } from "./MatchSession";
import { GameState } from "./dto/GameState";
import { PlayerInterface } from "./entities/player/PlayerInterface";

export class PlayerNotificationManager {

  async notifyGameState(game: DominoesGame) {
    const gameState: GameState = game.getGameState();
    const { players } = game;
    let promises: Promise<void>[] = players.map(player => player.notifyGameState(gameState));
    return await Promise.all(promises);
  }

  async notifyPlayersState(players: PlayerInterface[]) {
    let promises: Promise<void>[] = players.map(player => player.notifyPlayerState(player.getState()));
    return await Promise.all(promises);
  }


  async notifyMatchState(session: MatchSession) {
    const { players } = session;
    let promises: Promise<void>[] = players.map(player => player.notifyMatchState(session.getState()));
    return await Promise.all(promises);
  }

  async waitForPlayersAction(actionId: string, data: any = {}, players: PlayerInterface[]) {
    let promises: Promise<boolean>[] = players.map(player => player.waitForAction(actionId, data));
    return await Promise.all(promises);
  }

  async sendEventToPlayers(event: string, players: PlayerInterface[]) {
    let promises: Promise<void>[] = players.map(player => player.sendEvent(event));
    return await Promise.all(promises);
  }
}