import { PlayerInteractionInterface } from "../../PlayerInteractionInterface";
import { PlayerInteractionNetwork } from "../../PlayerInteractionNetwork";
import { PlayerHuman } from "./PlayerHuman";
import { NetworkClientNotifier } from "../../NetworkClientNotifier";
import { Tile } from "../Tile";
import { Board } from "../Board";
import { GameState } from "../../dto/GameState";
import { PlayerDto } from "../../dto/PlayerDto";
import { MatchSessionState } from "../../dto/MatchSessionState";
import { SocketDisconnectedError } from "../../../common/exceptions/SocketDisconnectedError";

export class NetworkPlayer extends PlayerHuman {
  socketId: string;
  playerInteraction: PlayerInteractionInterface = new PlayerInteractionNetwork(this);
  clientNotifier: NetworkClientNotifier = new NetworkClientNotifier();
  
  constructor(name: string, socketId: string) {
    super(name);
    this.socketId = socketId;
  }

  async notifyGameState(state: GameState): Promise<void> {
    const response = await this.clientNotifier.notifyPlayer(this, 'gameState', state);
    console.log('game state notified :>> ', response);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
  } 

  async notifyPlayerState(state: PlayerDto): Promise<void> {
    const response = await this.clientNotifier.notifyPlayer(this, 'playerState', state);
    console.log('player state notified :>> ', response);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
  }

  async notifyMatchState(state: MatchSessionState): Promise<void> {
    const response = await this.clientNotifier.notifyPlayer(this, 'matchState', state);
    console.log('session state notified :>> ', response);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
  }
  async waitForAction(actionId: string): Promise<boolean> {
    const response = await this.clientNotifier.notifyPlayer(this, actionId);
    if (response === undefined || response.status !== 'ok' ) {
      throw new SocketDisconnectedError();
    }
    const { actionResult } = response;
    return actionResult;
  }

  async sendEvent(event: string): Promise<void> {
    this.clientNotifier.sendEvent(this, event);
  }

  async chooseTile(board: Board): Promise<Tile> {
    return await this.playerInteraction.chooseTile(board);
  }
}