import { MatchSession } from "../../game/MatchSession";
import { NetworkPlayer } from "../../game/entities/player/NetworkPlayer";
import { SessionService } from "../services/SessionService";

import { ManagerBase } from "./ManagerBase";

export class SessionManager extends ManagerBase {
  private static sessions: any = {};
  private sessionService: SessionService = new SessionService();

  constructor() {
    super();
    this.logger.info('SessionController created');
  }

  createSession(data: any, socketId: string): any {
    const { user, sessionName } = data;
    const player =  new NetworkPlayer(user, socketId);
    const session = new MatchSession(player, sessionName);
    SessionManager.sessions[session.id] = session;
    this.sessionService.createSession(session);

    return {
      status: 'ok',
      sessionId: session.id,
      playerId: player.id
    };
  }

  joinSession(data: any, socketId: string): any {
    this.logger.debug('joinSession data :>> ')
    this.logger.object(data);
    const { user, sessionId } = data;
    const session: MatchSession = SessionManager.sessions[sessionId];
    const player =  new NetworkPlayer(user, socketId);
    session.addPlayer(player);
    this.sessionService.updateSession(session);
    return {
      status: 'ok',
      sessionId: session.id,
      playerId: player.id
    };
  }

  setPlayerReady(data: any): any {
    const { user, sessionId } = data;
    const session: MatchSession = SessionManager.sessions[sessionId];
    session.setPlayerReady(user)
  }

  startSession(data: any): any {
    const sessionId: string = data.sessionId;
    const seed: string | undefined = data.seed;
    const session = SessionManager.sessions[sessionId];

    if (!session) {
      return ({
        status: 'error',
        message: 'Session not found'
      });
    } else if (session.gameInProgress) {
      return {
        status: 'error',
        message: 'Game already in progress'
      };
    } else {
      const missingHumans = session.maxPlayers - session.numPlayers;
      for (let i = 0; i < missingHumans; i++) {
        session.addPlayer(session.createPlayerAI(i));
      }
      session.start(seed);
      return {
        status: 'ok'
      };
    }
  }


  getSession(id: string) {
    return SessionManager.sessions[id];
  }

  deleteSession(id: string) {
    delete SessionManager.sessions[id];
  }
}